#!/usr/bin/env bash

# Renames a user
# Required arguments: old user name, new user name

set -e

question() {
    echo
    echo "$@"
    read -r answer
    answer="${answer:0:1}"
    answer="${answer^}"
}

if [[ $# -ne 2 ]]; then
    echo "Usage: $0 old-user new-user"
    exit 1
fi

oldUser="$1"
newUser="$2"

# Make sure old user exists
if ! id "$oldUser" >/dev/null 2>&1; then
    echo "The user $oldUser does not exist."
    exit 1
fi

# Make sure old user is not logged in
if pgrep -u "$oldUser" >/dev/null; then
    echo "The user $oldUser is currently logged in or has running processes. Cannot continue."
    question "Would you like to forcibly log out $oldUser? (Y/N)"
    if [[ "$answer" != "Y" ]]; then
        exit 1
    fi
    systemctl stop display-manager
    loginctl terminate-user "$oldUser"
    sleep 2
    if pgrep -u "$oldUser" >/dev/null; then
        echo "The user $oldUser still has running processes after termination. Cannot continue."
        exit 1
    fi
fi

# Make sure new user does not exist
if id "$newUser" >/dev/null 2>&1; then
    echo "The user name $newUser is already taken."
    exit 1
fi

# Make sure the new name is acceptable
if ! [[ "$newUser" =~ ^[a-z_][a-z0-9_-]{0,31}$ ]]; then
    echo "$newUser is not an acceptable user name."
    exit 1
fi

# Passed all safety checks, proceed with the rename
echo "Renaming $oldUser to $newUser..."
groups=$(id -nG "$oldUser")
groups="${groups// /,}"
usermod -G "$groups" -m -d "/home/$newUser" -l "$newUser" "$oldUser"

# Update nodm.conf if it exists
if [[ -e /etc/nodm.conf ]]; then
    echo "Updating /etc/nodm.conf..."
    sed -i -e "s#^NODM_USER=.*#NODM_USER='$newUser'#" -e "s#^NODM_XSESSION=.*#NODM_XSESSION='/home/$newUser/.xinitrc'#" /etc/nodm.conf
fi

# Copy over crontab if it exists
if crontab -u "$oldUser" -l >/tmp/${oldUser}_crontab.bak 2>/dev/null; then
    echo "Copying over crontab..."
    if crontab -u "$newUser" "/tmp/${oldUser}_crontab.bak"; then
        rm "/tmp/${oldUser}_crontab.bak"
    else
        echo "Warning: failed to restore crontab for $newUser."
        rm "/tmp/${oldUser}_crontab.bak"
    fi
fi

# Update linger
if [[ -e "/var/lib/systemd/linger/$oldUser" ]]; then
    echo "Enabling linger..."
    mv "/var/lib/systemd/linger/$oldUser" "/var/lib/systemd/linger/$newUser"
fi

# Optionally update files in new home directory
echo
echo "Would you like to search for references to $oldUser and update them to $newUser in files located in /home/$newUser?"
question "This can take a while. (Y/N)"
if [[ "$answer" == "Y" ]]; then
    echo "Updating files..."
    find "/home/$newUser" -type f -exec grep -Iq . "{}" \; -and -exec sed -i "s|$oldUser|$newUser|g" "{}" \;
fi

echo
echo "Rename complete. The user, $newUser, is now available."
question "Would you like to reboot?"
if [[ "${answer}" == "Y" ]]; then
    reboot
fi

exit 0
